/*
  Title     : Person.cpp
  Version   : 1.0
  Copyright : 2004 Tero Hiekkalinna
  Author    : Tero Hiekkalinna (tero.hiekkalinna@helsinki.fi)
  Date      : 4.2.2004
*/

#include <sstream>
#include "Person.h"

// Constructors

Person::Person() { }

Person::Person(string fam, string per, string fat, string mot, string sex)
{
   this->familyid = fam;
   this->personid = per;
   this->fatherid = fat;
   this->motherid = mot;
   this->sex = sex;
   this->PedFileFormat = "post";
   this->istrait = false;
   this->isliab = false;
}

Person::Person(string fam, string per, string fat, string mot, string sex, string affection)
{
   this->familyid = fam;
   this->personid = per;
   this->fatherid = fat;
   this->motherid = mot;
   this->sex = sex;
   this->affection_status = affection;
   this->PedFileFormat = "post";
   this->istrait = false;
   this->isliab = false;
}

Person::Person(string fam, string per, string fat, string mot, string sex, string affection, string liab)
{
   this->familyid = fam;
   this->personid = per;
   this->fatherid = fat;
   this->motherid = mot;
   this->sex = sex;
   this->affection_status = affection;
   this->liability_class = liab;
   this->PedFileFormat = "post";
   this->istrait = false;
   this->isliab = false;
}

void Person::setFirstOffspring(string fos)
{
   this->first_offspring = fos;
}

void Person::setNextPaternalSibling(string nps)
{
   this->next_paternal_sibling = nps;
}

void Person::setNextMaternalSibling(string nms)
{
   this->next_maternal_sibling = nms;
}

void Person::setProbandStatus(string prob)
{
   this->proband_status = prob;
}

void Person::setAffectionStatus(string aff)
{
   this->affection_status = aff;
   this->istrait = true;
}

void Person::setLiabilityClass(string liab)
{
   this->liability_class = liab;
   this->isliab = true;
}

void Person::setPedFileFormat(string format)
{
   this->PedFileFormat = format;
}

// return familyid
string Person::getFamilyID()
{
   return this->familyid;
}

// return personid
string Person::getPersonID()
{
   return this->personid;
}

// return fatherid
string Person::getFatherID()
{
   return this->fatherid;
}

// return motherid
string Person::getMotherID()
{
   return this->motherid;
}

// return sex
string Person::getSex()
{
   return this->sex;
}

// return affection status
string Person::getAffectionStatus()
{
   return this->affection_status;
}

// return liability class
string Person::getLiabilityClass()
{
   return this->liability_class;
}

// inserts genotype
void Person::insertGenotype(string allele_1, string allele_2)
{
   Genotype gt = Genotype(allele_1,allele_2);
   this->genotype.push_back(gt);
}

string Person::getMarkerAllele(int m_index, int a_index)
{
   string allele = "NULL";
   for(unsigned int i = 0 ; i < genotype.size() ; i++)
   {
      if(i==m_index)
      {
         allele = this->genotype[i].getAllele(a_index);
         break;
      }
   }
   return allele;
}

// return string presentation of person node (ID fields and genotypes and phenotypes)
string Person::getInString()
{
   return getInString(false);
}

string Person::getInStringALL()
{
   return getInString(true);
}

string Person::getInString(bool all)
{
   ostringstream person_string;
   // person's info
   person_string << familyid << " " << personid << " " << fatherid << " " << motherid;
   if(this->PedFileFormat=="post")
   {
      person_string << " " << first_offspring << " " << next_paternal_sibling << " " << next_maternal_sibling << " " << next_maternal_sibling;
      person_string << " " << sex << " " << proband_status;
   }
   else
   {
      person_string << " " << sex;
   }
   if(this->istrait == true)
   {
      person_string << " " << affection_status;
   }
   if(this->isliab == true)
   {
      person_string << " " << liability_class;
   }
   if(all==true)
   {
      // genotypes
      for(unsigned int i = 0 ; i < genotype.size() ; i++)
      {
         person_string << " " << this->genotype[i].getGenotype();
      }
      person_string << endl;
   }
   return person_string.str();
}



