/*
  Title     : PedConvert.h
  Version   : 1.0
  Copyright : 2004 Tero Hiekkalinna
  Author    : Tero Hiekkalinna (tero.hiekkalinna@helsinki.fi)
  Date      : 4.2.2004

  58161-5 Tietorakenteiden Harjoitusty, kevt 2004
  Helsingin yliopisto, Tietojenksittelytieteen laitos
  Ohjaaja: Olaf Laczak
*/

#if !defined(PEDCONVERT_H)
   #define PEDCONVERT_H

#include <vector>

// Constants

class PedConvert
{
   private:

      char*  PedigreeFileName;
      char*  PhenotypeFileName;
      char*  MapFileName;
      char*  ModelFileName;
      int    numberOfMarkers;
      int    numberOfPhenotypes;
      bool   sexCovariate;
      bool   ignorePhenotype;
      bool   allTraits;
      bool   discrete;
      string gender;
      double disable_map_position;
      string program;
      bool   ModelFileIsSet;

      vector<string> phenotypename;
      vector<string> markername;

      char*  OutputPedigreeFileName;
      char*  OutputLocusFileName;
      char*  OutputMapFileName;
      char*  OutputPhenotypeFileName;
      char*  OutputMarkerFileName;

   public:

      PedConvert();

      void setPedigreeFile(char *);
      void setPhenotypeFile(char *);
      void setMapFile(char *);
      void setModelFile(char *);
      bool setNumberOfMarkers(int);
      bool setNumberOfPhenotypes(int);
      void addPhenotypeName(string);
      void addMarkerName(string);
      void setSexCovariate();
      void setGender(string);
      void setIgnoreTrait();
      void setAllTraits();
      void setProgram(string);

      void setOutputPedigreeFile(char *);
      void setOutputLocusFile(char *);
      void setOutputMapFile(char *);
      void setOutputPhenotypeFile(char *);
      void setOutputMarkerFile(char *);

      char* getPedigreeFileName();
      char* getPhenotypeFileName();
      char* getMapFileName();
      char* getModelFileName();
      int   getNumberOfMarkers();
      int   getNumberOfPhenotypes();
      int   getNumberOfMarkersInMapFile();
      string getPhenotypeName(int);
      string getMarkerName(int);
      bool  isSexCovariate();
      bool  isIgnorePhenotype();
      bool  areAllTraits();
      bool  isDiscrete();
      string getGender();
      double getDisableMapPosition();
      string getProgram();
      bool  isModelFileSet();

      char* getOutputPedigreeFile();
      char* getOutputLocusFile();
      char* getOutputMapFile();
      char* getOutputPhenotypeFile();
      char* getOutputMarkerFile();

      bool writeLinkageFormatLocusFile();
      bool writePedCheckNamesFile();

      bool parseArguments(int,char **);

      void welcome();
      void usage();

      void print();

      

};

#endif
