/*
  Title     : Pedigree.cpp
  Version   : 1.0
  Copyright : 2004 Tero Hiekkalinna
  Author    : Tero Hiekkalinna (tero.hiekkalinna@helsinki.fi)
  Date      : 4.2.2004

  58161-5 Tietorakenteiden Harjoitusty, kevt 2004
  Helsingin yliopisto, Tietojenksittelytieteen laitos
  Ohjaaja: Olaf Laczak
*/

#include <iostream>
using namespace std;

#include "Pedigree.h"

// Constructors

Pedigree::Pedigree(){}

Pedigree::Pedigree(string key)
{
   this->root = new Person("NIL","BLACK");
   this->key = key;
}

Pedigree::Pedigree(string key, string color)
{
   this->root = new Person("NIL","BLACK");
   this->key = key;
   this->color = color;
}

// return key (personid)
string Pedigree::getKey()
{
   return this->key;
}

// set left node
void Pedigree::setLeft(Pedigree *ped)
{
   this->left = ped;
}

// set right node
void Pedigree::setRight(Pedigree *ped)
{
   this->right = ped;
}

// set parent node
void Pedigree::setParent(Pedigree *ped)
{
   this->parent = ped;
}

// set node to red
void Pedigree::setRED()
{
   this->color = "RED";
}

// set node to black
void Pedigree::setBLACK()
{
    this->color = "BLACK";
}

// return node color
string Pedigree::getColor()
{
    return this->color;
}

// return left node
Pedigree* Pedigree::getLeft()
{
   return this->left;
}

// return right node
Pedigree* Pedigree::getRight()
{
   return this->right;
}

// return parent node
Pedigree* Pedigree::getParent()
{
   return this->parent;
}

// return true if node is nil, else false
bool Pedigree::isNIL()
{
   if(this->key == "NIL")
      return true;
   else
      return false;
}

// insert person node
bool Pedigree::insert(Person *per)
{
   return RBInsert(per);
}

/*
 Red-Black Tree methods

 Variable names (z,x,y) are same as in "Indroduction to
 Algorithms", by Thomas H Cormer et al page 278-281, so it's
 easyer to follow with book if one wishes.

*/

// RB-Insert
bool Pedigree::RBInsert(Person *z)
{
   Person *y = new Person("NIL","BLACK");
   Person *x = root;

   while(!x->isNIL())
   {
      /*
       * Let's check that we don't insert duplicate person to pedigree
       */
      if(z->getKey() == x->getKey())
      {
         cout << "Error: Duplicate person " << z->getKey() << " in family " << z->getFamilyID() << "." << endl;
         return false;
      }

      y = x;

      if(z->getKey() < x->getKey())
      {
         x = x->getLeft();
      }
      else
      {
         x = x->getRight();
      }
   }

   z->setParent(y);

   if(y->isNIL())
   {
      this->root = z;
   }
   else
   {
      if(z->getKey() < y->getKey())
      {
         y->setLeft(z);
      }
      else
      {
         y->setRight(z);
      }
   }

   // set Red-Black Tree properties
   z->setLeft(new Person("NIL","BLACK"));
   z->setRight(new Person("NIL","BLACK"));
   z->setRED();

   // fixup
   RBInsertFixup(z);

   return true;
}

// RB-Insert-Fixup
void Pedigree::RBInsertFixup(Person *z)
{
   while((z->getParent())->getColor() == "RED")
   {
      Person *parent      = z->getParent();
      Person *grandparent = parent->getParent();

      if(parent == grandparent->getLeft())
      {
         Person *y = grandparent->getRight();

         // z's uncle is red
         if(y->getColor() == "RED")
         {
            parent->setBLACK();
            y->setBLACK();
            grandparent->setRED();
            z = grandparent;
         }
         else
         {
            // z's uncle y is black and z is a right child
            if(z == parent->getRight())
            {
               z = z->getParent();
               LeftRotate(z);
            }

            /* Let's get parent and grandparent pointers again
             * to make sure that they are correct after LeftRotate(z) */
            parent = z->getParent();
            grandparent = parent->getParent();

            // z's uncle y is black and z's is a left child
            parent->setBLACK();
            grandparent->setRED();
            RightRotate(grandparent);
         }
      }
      else
      {
         Person *y = grandparent->getLeft();

         // z's uncle y is red
         if(y->getColor() == "RED")
         {
            parent->setBLACK();
            y->setBLACK();
            grandparent->setRED();
            z = grandparent;
         }
         else
         {
            // z's uncle is black and z is a left child
            if(z == parent->getLeft())
            {
               z = z->getParent();
               RightRotate(z);
            }

            /* Let's get parent and grand parenr pointers again
             * to make sure that they are correct after RightRotate(z) */
            parent = z->getParent();
            grandparent = parent->getParent();

            // z's uncle is black and z is a right child
            parent->setBLACK();
            grandparent->setRED();
            LeftRotate(grandparent);
         }

      }
   }
   root->setBLACK();
}

/*  Left-Rotate

   X                Y
  / \              / \
 1   Y     -->    X   3
    / \          / \
   2   3        1   2

*/
void Pedigree::LeftRotate(Person *x)
{
   Person *y = x->getRight();

   // y's left is now x's right
   x->setRight(y->getLeft());

   if(!(y->getLeft())->isNIL())
   {
      Person *left = y->getLeft();
      left->setParent(x);
   }

   // x's parent is now y's parent
   y->setParent(x->getParent());

   if((x->getParent())->isNIL())
   {
      this->root = y;
   }
   else
   {
      Person *w = x->getParent();
      if(x == w->getLeft())
      {
         w->setLeft(y);
      }
      else
      {
         w->setRight(y);
      }
   }
   // y's left in now x
   y->setLeft(x);
   // x's parent is now y
   x->setParent(y);
}

/*  Right-Rotate

   X                Y
  / \              / \
 1   Y     <--    X   3
    / \          / \
   2   3        1   2

*/
void Pedigree::RightRotate(Person *x)
{
   Person *y = x->getLeft();

   // x's left is now y's right
   x->setLeft(y->getRight());

   if(!(y->getRight())->isNIL())
   {
      Person *right = y->getRight();
      right->setParent(x);
   }

   // y's pareant is now x's parent
   y->setParent(x->getParent());

   if((x->getParent())->isNIL())
   {
      this->root = y;
   }
   else
   {
      Person *w = x->getParent();
      if(x == w->getLeft())
      {
         w->setLeft(y);
      }
      else
      {
         w->setRight(y);
      }
   }
   // y's right is now x
   y->setRight(x);
   // x's parent is y
   x->setParent(y);
}

// finds person with key (personid) if not found returns NULL
Person* Pedigree::find(string key)
{
   Person *x = root;

   while(!x->isNIL())
   {
      if(x->getKey() == key)
         return x;

      if(key < x->getKey())
      {
         x = x->getLeft();
      }
      else
      {
         x = x->getRight();
      }
   }
   return (Person *) NULL;
}

// count size of the pedigree
int Pedigree::size()
{
   this->pedigree_size = 0;
   size(this->root);
   return this->pedigree_size;
}

// count size of pedigree, used by size() method
void Pedigree::size(Person *root)
{
   if(root->isNIL())
   {
      return;
   }
   else
   {
      size(root->getRight());
      this->pedigree_size += 1;
      size(root->getLeft());
   }

}

// prints pedigree, used for debugging
void Pedigree::print()
{
   RBprint(this->root,"");
}

// prints pedigree, used for debugging
void Pedigree::RBprint(Person *root, string s)
{
   if(root->isNIL())
   {
      return;
   }
   else
   {
      RBprint(root->getRight(), s + "     |");
      cout << endl;
      cout << s << "---->" << root->getKey() << " (" << root->getColor() << ")";
      cout << endl;
      RBprint(root->getLeft(),  s + "     |");
   }
}

// return string presentation of pedigrees
string Pedigree::getInString()
{
   this->ped_string = "";
   getInString(root);
   return this->ped_string;
}

// makes string presentation of pedigree
void Pedigree::getInString(Person *root)
{
   if(root->isNIL())
   {
      return;
   }
   else
   {
      getInString(root->getLeft());
      this->ped_string += root->getInString();
      getInString(root->getRight());
   }
}

string Pedigree::getInSolarString()
{
   this->ped_string = "";
   getInSolarString(root);
   return this->ped_string;
}

void Pedigree::getInSolarString(Person *root)
{
   if(root->isNIL())
   {
      return;
   }
   else
   {
      getInSolarString(root->getLeft());
      this->ped_string += root->getInSolarString();
      getInSolarString(root->getRight());
   }
}

string Pedigree::getMarkersInSolarString()
{
   this->ped_string = "";
   getMarkersInSolarString(root);
   return this->ped_string;
}

void Pedigree::getMarkersInSolarString(Person *root)
{
   if(root->isNIL())
   {
      return;
   }
   else
   {
      getMarkersInSolarString(root->getLeft());
      this->ped_string += root->getMarkersInSolarString();
      getMarkersInSolarString(root->getRight());
   }
}

string Pedigree::getPhenotypesInSolarString()
{
   this->ped_string = "";
   getPhenotypesInSolarString(root);
   return this->ped_string;
}

void Pedigree::getPhenotypesInSolarString(Person *root)
{
   if(root->isNIL())
   {
      return;
   }
   else
   {
      getPhenotypesInSolarString(root->getLeft());
      this->ped_string += root->getPhenotypesInSolarString();
      getPhenotypesInSolarString(root->getRight());
   }
}

/*
 Linked list testing methods
*/

// sets next pedigree in linked list
void Pedigree::setNext(Pedigree *ped)
{
   this->next = ped;
}

// return next pedigree in linked list
Pedigree* Pedigree::getNext()
{
   return this->next;
}
