/*
  Title     : PedigreeTree.h
  Version   : 1.0
  Copyright : 2004 Tero Hiekkalinna
  Author    : Tero Hiekkalinna (tero.hiekkalinna@helsinki.fi)
  Date      : 4.2.2004

  58161-5 Tietorakenteiden Harjoitusty, kevt 2004
  Helsingin yliopisto, Tietojenksittelytieteen laitos
  Ohjaaja: Olaf Laczak
*/

#include <sstream>
using namespace std;

#if !defined(PEDIGREETREE_H)
   #define PEDIGREETREE_H

#include "Pedigree.h"
#include "PedConvert.h"

class PedigreeTree
{
   private:

      // Red-black Tree root
      Pedigree *root;

      int      total_pedigrees;
      int      total_individuals;

      // all pedigrees are printed to this string stream when
      // printing out all pedigrees to merlin format
      ostringstream  pedigrees_string;
      ostringstream  solarmarker_string;
      ostringstream  solarphenotype_string;

      PedConvert *MetaData;

      // Methods for Pedigree Red-Black Tree
      bool RBInsert(Pedigree *);
      void RBInsertFixup(Pedigree *);
      void LeftRotate(Pedigree *);
      void RightRotate(Pedigree *);
      void RBprint(Pedigree *, string);
      void size(Pedigree *);
      void sizeTotal(Pedigree *);

      void writeLinkageFormat(Pedigree *);
      void writeSolarFormatPedigree(Pedigree *);
      void writeSolarFormatMarkerFile(Pedigree *);
      void writeSolarFormatPhenotypeFile(Pedigree *);

   public:

      PedigreeTree();
      bool insert(Pedigree *);
      Pedigree* find(string);
      int size();
      int sizeTotal();
      void print();
      void setMetaData(PedConvert *);
      void writeLinkageFormat();
      void writeSolarFormatPedigree();
      void writeSolarFormatMarkerFile();
      void writeSolarFormatPhenotypeFile();

};

#endif
