/*
  Title     : Person.cpp
  Version   : 1.0
  Copyright : 2004 Tero Hiekkalinna
  Author    : Tero Hiekkalinna (tero.hiekkalinna@helsinki.fi)
  Date      : 4.2.2004

  58161-5 Tietorakenteiden Harjoitusty, kevt 2004
  Helsingin yliopisto, Tietojenksittelytieteen laitos
  Ohjaaja: Olaf Laczak
*/

#include <string>
using namespace std;
#include <sstream>
#include <iostream>

#include "Person.h"

#define MERLIN_MISSING_VALUE  "x"
#define QUALITATIVE_MISSING_VALUE "0"

// Constructors

Person::Person() { }

Person::Person(string per, string color)
{
   this->personid = per;
   this->color = color;
   this->key = per;

   this->gt_head = NULL;
   this->gt_tail = NULL;
   this->ph_head = NULL;
   this->ph_tail = NULL;
}

Person::Person(string fam, string per, string fat, string mot, string sex)
{
   this->familyid = fam;
   this->personid = per;
   this->fatherid = fat;
   this->motherid = mot;
   this->sex = sex;
   this->key = per;

   this->gt_head = NULL;
   this->gt_tail = NULL;
   this->ph_head = NULL;
   this->ph_tail = NULL;
}

Person::Person(string fam, string per, string fat, string mot, string sex, string affection)
{
   this->familyid = fam;
   this->personid = per;
   this->fatherid = fat;
   this->motherid = mot;
   this->sex = sex;
   this->affection_status = affection;
   this->key = per;

   this->gt_head = NULL;
   this->gt_tail = NULL;
   this->ph_head = NULL;
   this->ph_tail = NULL;
}

Person::Person(string fam, string per, string fat, string mot, string sex, string affection, string liab)
{
   this->familyid = fam;
   this->personid = per;
   this->fatherid = fat;
   this->motherid = mot;
   this->sex = sex;
   this->affection_status = affection;
   this->liability_class = liab;
   this->key = per;

   this->gt_head = NULL;
   this->gt_tail = NULL;
   this->ph_head = NULL;
   this->ph_tail = NULL;
}

// return familyid
string Person::getFamilyID()
{
   return this->familyid;
}

// return personid
string Person::getPersonID()
{
   return this->personid;
}

// return fatherid
string Person::getFatherID()
{
   return this->fatherid;
}

// return motherid
string Person::getMotherID()
{
   return this->motherid;
}

// return sex
string Person::getSex()
{
   return this->sex;
}

// inserts genotype to linked list
void Person::insertGenotype(string allele_1, string allele_2)
{
   Genotype *gt = new Genotype(allele_1,allele_2);
   gt->setNext(NULL);

   if(gt_head==NULL)
   {
      gt_head = gt_tail = gt;
      return;
   }
   else
   {
      gt_tail->setNext(gt);
      gt_tail = gt;
   }
}

// insert phenotype to linked list
bool Person::insertPhenotype(string name, string value)
{
   Phenotype *ph = new Phenotype(name,value);
   ph->setNext(NULL);

   /* check if phenotype if inserted */
   Phenotype *ph_check = ph_head;
   while(ph_check!=NULL)
   {
      if(name == ph_check->getName()){
         return false;
      }
      ph_check = ph_check->getNext();
   }

   if(ph_head==NULL)
   {
      ph_head = ph_tail = ph;
   }
   else
   {
      ph_tail->setNext(ph);
      ph_tail = ph;
   }

   return true;
}

// returns key (familyid)
string Person::getKey()
{
   return this->key;
}

// set left node
void Person::setLeft(Person *per)
{
   this->left = per;
}

// set right node
void Person::setRight(Person *per)
{
   this->right = per;
}

// set parent node
void Person::setParent(Person *per)
{
   this->parent = per;
}

// set node to red
void Person::setRED()
{
   this->color = "RED";
}

// set node to black
void Person::setBLACK()
{
    this->color = "BLACK";
}

// returns node color
string Person::getColor()
{
    return this->color;
}

// return left node
Person* Person::getLeft()
{
   return this->left;
}

// return right node
Person* Person::getRight()
{
   return this->right;
}

// returns node's parent
Person* Person::getParent()
{
   return this->parent;
}

// return true if node is nil
bool Person::isNIL()
{
   if(this->key == "NIL")
      return true;
   else
      return false;
}

// return string presentation of person node (ID fields and genotypes and phenotypes)
string Person::getInString()
{
   ostringstream person_string;

   // person's info
   person_string << familyid << " " << personid << " " << fatherid << " " << motherid << " " << sex;

   // person's phenotypes
   Phenotype *ph = ph_head;

   // if phenotype list is null, print 'x' (MERLIN_MISSING_VALUE) for phenotype
   if(ph==NULL || (sex == "1" && MetaData->getGender() == "females") || (sex == "2" && MetaData->getGender() == "males"))
   {
      for(int i = 0 ; i < MetaData->getNumberOfPhenotypes() ; i++)
      {
         if(MetaData->isDiscrete()==true)
         {
            person_string << " " << QUALITATIVE_MISSING_VALUE;
         }
         else
         {
            person_string << " " << MERLIN_MISSING_VALUE;
         }
      }
   }
   else
   {
      while(ph!=NULL)
      {
         person_string << " " << ph->get();
         ph = ph->getNext();
      }
   }

   // Add sex as covariate
   if(MetaData->isSexCovariate() && MetaData->getGender() == "all")
   {
      person_string << " " << sex;
   }

   // person's genotypes
   Genotype *gt = gt_head;
   while(gt!=NULL)
   {
      person_string << " " << gt->get();
      gt = gt->getNext();
   }

   person_string << endl;

   return person_string.str();
}

string Person::getInSolarString()
{
   ostringstream person_string;

   // person's info
   person_string << familyid << "," << personid << "," << fatherid << "," << motherid << "," << sex << endl;
   return person_string.str();
}

string Person::getPhenotypesInSolarString()
{
   ostringstream person_string;

   // person's info
   person_string << familyid << "," << personid;

   // person's phenotypes
   Phenotype *ph = ph_head;

   // if phenotype is null of analysis is other sex only, return
   if(ph==NULL || (sex == "1" && MetaData->getGender() == "females") || (sex == "2" && MetaData->getGender() == "males"))
   {
      return "";
   }
   else
   {
      while(ph!=NULL)
      {
         if(ph->get() == "x" || ph->get() == "X" || ph->get() == "-")
         {
            person_string << ",";
         }
         else
         {
            person_string << "," << ph->get();
         }
         ph = ph->getNext();
      }
   }
   person_string << endl;
   return person_string.str();
}

// return string presentation of person node (ID fields and genotypes and phenotypes)
string Person::getMarkersInSolarString()
{
   ostringstream person_string;

   // person's info
   person_string << familyid << "," << personid;

   // person's genotypes
   Genotype *gt = gt_head;
   while(gt!=NULL)
   {
      person_string << "," << gt->getSolarGenotype();
      gt = gt->getNext();
   }

   person_string << endl;

   return person_string.str();
}


// print person, only testing for testing
void Person::print()
{
   // person's info
   cout << familyid << " " << personid << " " << fatherid << " " << motherid << " " << sex;

   // person's genotypes
   Genotype *gt = gt_head;
   while(gt!=NULL)
   {
      cout << "  " << gt->get();
      gt = gt->getNext();
   }

   // person's phenotypes
   Phenotype *ph = ph_head;
   while(ph!=NULL)
   {
      cout << "  " << ph->get();
      ph = ph->getNext();
   }
   cout << endl;
}


// set metadata
void Person::setMetaData(PedConvert *MetaData)
{
   this->MetaData = MetaData;
}
