/*
  Title     : Person.h
  Version   : 1.0
  Copyright : 2004 Tero Hiekkalinna
  Author    : Tero Hiekkalinna (tero.hiekkalinna@helsinki.fi)
  Date      : 4.2.2004

  58161-5 Tietorakenteiden Harjoitusty, kevt 2004
  Helsingin yliopisto, Tietojenksittelytieteen laitos
  Ohjaaja: Olaf Laczak
*/

#if !defined(PERSON_H)
   #define PERSON_H

#include "Genotype.h"
#include "Phenotype.h"
#include "PedConvert.h"

class Person
{
   private:

      // person information: familyid, personid, etc ...
      string   familyid;
      string   personid;
      string   fatherid;
      string   motherid;
      string   sex;

      // trait
      string   affection_status;
      string   liability_class;

      // MetaData info, number of phenotypes, etc...
      PedConvert *MetaData;

      // Genotypes and Phenotypes will be stored in linked list
      Genotype *gt_head;
      Genotype *gt_tail;

      Phenotype *ph_head;
      Phenotype *ph_tail;

      // Red-Black Tree variables
      string   key;     // Person ID
      string   color;
      Person   *left;
      Person   *right;
      Person   *parent;

   public:

      Person();
      Person(string,string);
      Person(string,string,string,string,string);
      Person(string,string,string,string,string,string);
      Person(string,string,string,string,string,string,string);

      string getFamilyID();
      string getPersonID();
      string getFatherID();
      string getMotherID();
      string getSex();

      void insertGenotype(string,string);
      bool insertPhenotype(string,string);

      // Red-Black Tree methods for Pedigree
      string getKey();
      void setLeft(Person *);
      void setRight(Person *);
      void setParent(Person *);
      void setRED();
      void setBLACK();
      string getColor();
      Person* getLeft();
      Person* getRight();
      Person* getParent();
      bool isNIL();

      string getInString();
      string getInSolarString();
      string getPhenotypesInSolarString();
      string getMarkersInSolarString();

      void print();

      void setMetaData(PedConvert *);
};

#endif
