/*
  Title     : TestPedConvert.cpp
  Version   : 1.0
  Copyright : 2004 Tero Hiekkalinna
  Author    : Tero Hiekkalinna (tero.hiekkalinna@helsinki.fi)
  Date      : 4.2.2004

  58161-5 Tietorakenteiden Harjoitusty, kevt 2004
  Helsingin yliopisto, Tietojenksittelytieteen laitos
  Ohjaaja: Olaf Laczak
*/

#include <iostream>
using namespace std;
#include <fstream>

#include "TestPedConvert.h"
#include "PedigreeTree.h"
#include "Pedigree.h"
#include "Person.h"
#include "Genotype.h"
#include "Phenotype.h"
#include "Utils.h"

/*
 Test Genotype class
*/
void test_Genotype()
{
   Genotype *gt      = new Genotype("1","2");
   Genotype *gt_next = new Genotype("3","4");

   gt->setNext(gt_next);

   cout << endl;
   cout << "######### TEST GENOTYPE ##########" << endl;
   cout << endl;
   cout << "Genotype:      " << gt->get() << endl;
   cout << "Genotype next: " << (gt->getNext())->get() << endl;
}

/*
 Test Phenotype class
*/
void test_Phenotype()
{
   Phenotype *ph      = new Phenotype("PHEN_BMI","34.3");
   Phenotype *ph_next = new Phenotype("PHEN_HEIGHT","190");

   ph->setNext(ph_next);

   cout << endl;
   cout << "######### TEST PHENOTYPE ##########" << endl;
   cout << endl;
   cout << "Phenotype:      " << ph->get() << endl;
   cout << "Phenotype next: " << (ph->getNext())->get() << endl;
}

/*
 Test Person class
*/
void test_Person()
{
   Person *dude = new Person("Doe","John","0","0","M");

   dude->insertGenotype("1","2");
   dude->insertPhenotype("BMI","29.67");

   cout << endl;
   cout << "######## TEST PERSON #########" << endl;
   cout << dude->getInString() << endl;

   dude->insertGenotype("3","4");
   dude->insertPhenotype("HDL","1.0");

   cout << endl;
   cout << "######## TEST PERSON #########" << endl;
   dude->print();
}

/*
 Test Pedigree class
*/
void test_Pedigree()
{
   Pedigree *pedigree = new Pedigree("TestPed");
   Person *person = NULL;

   if(pedigree == NULL)
   {
      cout << "TEST ERROR: pedigree null!" << endl;
      return;
   }

   string per;
   for(int personid = 1 ; personid <= 20 ; personid++)
   {
      per = "per" + IntToString(personid);
      person = new Person("TestPed",per,"empty","empty","M");
      if(person==NULL)
      {
         cout << "TEST ERROR: person null!" << endl;
      }
      else
      {
         pedigree->insert(person);
      }
   }

   cout << endl;
   cout << "######## TEST PEDIGREE #########" << endl;
   pedigree->print();
}

/*
 Test PedigreeTree class
*/
void test_PedigreeTree()
{
   PedigreeTree *kainuu = new PedigreeTree();

   /*

    Insert few pedigrees to PedigreeTree and print pedigrees and pedigree tree after each insert

    Test pedigrees which has names are my own friends or my own relatives

    NO REAL DATA OVER HERE!!!!

   */

   cout << "############ KAINUU TEST ##############" << endl;
   cout << endl;

   // hiekkalinna
   Pedigree *hiekkalinna = new Pedigree("Hiekkalinna");
   Person *matti    = new Person("Hiekkalinna","Matti","0","0","M");
   Person *kaija    = new Person("Hiekkalinna","Kaija","0","0","F");
   Person *pauliina = new Person("Hiekkalinna","Pauliina","Matti","Kaija","F");
   Person *ukko     = new Person("Hiekkalinna","Ukko","Tero","Pauliina","M");
   Person *tapani   = new Person("Hiekkalinna","Tapani","0","0","M");
   Person *anneli   = new Person("Hiekkalinna","Anneli","0","0","F");
   Person *tero     = new Person("Hiekkalinna","Tero","Tapani","Anneli","M");
   hiekkalinna->insert(matti);
   hiekkalinna->insert(kaija);
   hiekkalinna->insert(pauliina);
   hiekkalinna->insert(ukko);
   hiekkalinna->insert(tapani);
   hiekkalinna->insert(anneli);
   hiekkalinna->insert(tero);
   hiekkalinna->print();
   kainuu->insert(hiekkalinna);

   cout << endl;
   cout << "Pedigree Red-Black tree: " << endl;
   kainuu->print();
   cout << "Number of pedigrees in RB tree: " << kainuu->size() << endl;
   cout << "Number of individuals in RB tree (TOTAL): " << kainuu->sizeTotal() << endl;

   // juntunen
   Pedigree *juntunen = new Pedigree("Juntunen");
   tapani   = new Person("Juntunen","Tapani","0","0","M");
   anneli   = new Person("Juntunen","Anneli","0","0","F");
   Person *tommi  = new Person("Juntunen","Tommi","Tapani","Anneli","M");
   tero     = new Person("Juntunen","Tero","Tapani","Anneli","M");
   juntunen->insert(tapani);
   juntunen->insert(anneli);
   juntunen->insert(tommi);
   juntunen->insert(tero);
   juntunen->print();
   kainuu->insert(juntunen);

   cout << endl;
   cout << "Pedigree Red-Black tree: " << endl;
   kainuu->print();
   cout << "Number of pedigrees in RB tree: " << kainuu->size() << endl;
   cout << "Number of individuals in RB tree (TOTAL): " << kainuu->sizeTotal() << endl;

   // kantola
   Pedigree *kantola = new Pedigree("Kantola");
   Person *raimo   = new Person("Kantola","Raimo","0","0","M");
   Person *hannele = new Person("Kantola","Hannele","0","0","F");
   Person *jute    = new Person("Kantola","Jute","Raimo","Hannele","M");
   Person *jouni   = new Person("Kantola","Jouni","Raimo","Hannele","M");
   Person *ilaria  = new Person("Kantola","Ilaria","0","0","F");
   Person *miro    = new Person("Kantola","Miro","Jouni","Ilaria","M");
   kantola->insert(raimo);
   kantola->insert(hannele);
   kantola->insert(jute);
   kantola->insert(jouni);
   kantola->insert(ilaria);
   kantola->insert(miro);
   kantola->print();
   kainuu->insert(kantola);

   cout << endl;
   cout << "Pedigree Red-Black tree: " << endl;
   kainuu->print();
   cout << "Number of pedigrees in RB tree: " << kainuu->size() << endl;
   cout << "Number of individuals in RB tree (TOTAL): " << kainuu->sizeTotal() << endl;

   // sood
   Pedigree *sood = new Pedigree("Sood");
   Person *kismat = new Person("Sood","Kismat","0","0","M");
   Person *aimee  = new Person("Sood","Aimee","0","0","F");
   Person *kid    = new Person("Sood","Kid","Kismat","Aimee","M");
   sood->insert(kismat);
   sood->insert(aimee);
   sood->insert(kid);
   sood->print();
   kainuu->insert(sood);

   cout << "Pedigree Red-Black tree: " << endl;
   kainuu->print();
   cout << "Number of pedigrees in RB tree: " << kainuu->size() << endl;
   cout << "Number of individuals in RB tree (TOTAL): " << kainuu->sizeTotal() << endl;

   // anttila
   Pedigree *anttila = new Pedigree("Anttila");
   Person *a         = new Person("Anttila","a","0","0","M");
   Person *b         = new Person("Anttila","b","0","0","F");
   Person *c         = new Person("Anttila","c","a","b","M");
   anttila->insert(a);
   anttila->insert(b);
   anttila->insert(c);
   anttila->print();
   kainuu->insert(anttila);

   cout << endl;
   cout << "Pedigree Red-Black tree: " << endl;
   kainuu->print();
   cout << "Number of pedigrees in RB tree: " << kainuu->size() << endl;
   cout << "Number of individuals in RB tree (TOTAL): " << kainuu->sizeTotal() << endl;

   //
   Pedigree *bujaa = new Pedigree("Bujaa");
   Person *ba         = new Person("Bujaa","a","0","0","M");
   Person *bb         = new Person("Bujaa","b","a","b","M");
   Person *bc         = new Person("Bujaa","c","a","b","M");
   bujaa->insert(ba);
   bujaa->insert(bb);
   bujaa->insert(bc);
   bujaa->print();
   // Duplicate insert!!!
   cout << "Testing duplicate insert to pedigree..." << endl;
   bujaa->insert(bc);
   kainuu->insert(bujaa);

   cout << endl;
   cout << "Pedigree Red-Black tree: " << endl;
   kainuu->print();
   cout << "Number of pedigrees in RB tree: " << kainuu->size() << endl;
   cout << "Number of individuals in RB tree (TOTAL): " << kainuu->sizeTotal() << endl;

   cout << "Testing duplicate insert to pedigree tree, but new person..." << endl;
   Pedigree *hiekkal = new Pedigree("Hiekkalinna");
   Person *ukko2 = new Person("Hiekkalinna","Ukko2","Tero","Pauliina","M");
   bool insertOK = kainuu->insert(hiekkal);
   if(insertOK == true)
   {
      hiekkal->insert(ukko2);
   }
   else
   {
      Pedigree *old = kainuu->find("Hiekkalinna");
      if(old!=NULL)
      {
         old->insert(ukko2);
      }
   }

   cout << endl;
   cout << "Pedigree Red-Black tree: " << endl;
   kainuu->print();
   cout << "Number of pedigrees in RB tree: " << kainuu->size() << endl;
   cout << "Number of individuals in RB tree (TOTAL): " << kainuu->sizeTotal() << endl;


   // Alphabeth
   cout << "############ Alphabeth TEST ##############" << endl;
   cout << endl;

   PedigreeTree *tree = new PedigreeTree();
   tree->insert(new Pedigree("A"));
   tree->insert(new Pedigree("B"));
   tree->insert(new Pedigree("C"));
   tree->insert(new Pedigree("D"));
   tree->insert(new Pedigree("E"));
   tree->insert(new Pedigree("F"));
   tree->insert(new Pedigree("G"));
   tree->insert(new Pedigree("H"));
   tree->insert(new Pedigree("I"));
   tree->insert(new Pedigree("J"));
   tree->insert(new Pedigree("K"));
   tree->insert(new Pedigree("L"));
   tree->insert(new Pedigree("M"));
   tree->insert(new Pedigree("N"));
   tree->insert(new Pedigree("O"));
   tree->insert(new Pedigree("P"));
   tree->insert(new Pedigree("Q"));
   tree->insert(new Pedigree("R"));
   tree->insert(new Pedigree("S"));
   tree->insert(new Pedigree("T"));
   tree->insert(new Pedigree("U"));
   tree->insert(new Pedigree("V"));
   tree->insert(new Pedigree("X"));
   tree->insert(new Pedigree("Y"));
   tree->insert(new Pedigree("Z"));
   tree->insert(new Pedigree("AA"));
   tree->insert(new Pedigree("BB"));
   tree->insert(new Pedigree("AAA"));
   tree->insert(new Pedigree("BBB"));
   tree->insert(new Pedigree("AAAA"));
   tree->insert(new Pedigree("BBBB"));
   tree->print();

   cout << endl;
   cout << "Pedigree Red-Black tree: " << endl;
   kainuu->print();
   cout << "Number of pedigrees in RB tree: " << kainuu->size() << endl;
   cout << "Number of individuals in RB tree (TOTAL): " << kainuu->sizeTotal() << endl;
}

/*

  Make test input files for pedconvert, arguments are in this order:

   number of pedigrees
   number of persons in a pedigree
   number of markers
   number of phenotypes
   pedigree file name
   phenotype file name

*/
void write_test_input_files(int nped, int nind, int nmar, int nphen, const char *pedfile, const char *phenfile)
{
   ofstream pedout(pedfile);
   if(!pedout)
   {
      cout << "Error: Unable to open write to file " << pedfile << endl;
      return;
   }

   ofstream phenout(phenfile);
   if(!phenout)
   {
      cout << "Error: Unable to open write to file " << phenfile << endl;
      return;
   }

   phenout << nphen << "   ";
   for(int phen_index = 1 ; phen_index <= nphen ; phen_index++)
   {
      phenout << "   Phen_" << phen_index;
   }
   phenout << endl;

   for(int ped_index = 1 ; ped_index <= nped ; ped_index++)
   {
      /*
       *  PEDIGREE FILE
       */

      // Father
      pedout << ped_index << "  1  0  0  1  1 ";
      for(int nmar_index = 0 ; nmar_index < nmar ; nmar_index++)
      {
         pedout << "  1 2  ";
      }
      pedout << endl;

      // Mother
      pedout << ped_index << "  2  0  0  2  1 ";
      for(int nmar_index = 0 ; nmar_index < nmar ; nmar_index++)
      {
         pedout << "  1 2  ";
      }
      pedout << endl;

      // Kids
      for(int nkids_index = 3 ; nkids_index <= nind ; nkids_index++)
      {
         pedout << ped_index << "  " << nkids_index << "  1  2  2  2 ";
         for(int nmar_index = 0 ; nmar_index < nmar ; nmar_index++)
         {
            pedout << "  1 2  ";
         }
         pedout << endl;
      }

      /*
       * PHENOTYPE FILE
       */

      for(int nind_index = 1 ; nind_index <= nind ; nind_index++)
      {
         phenout << ped_index << "  " << nind_index;
         for(int nphen_index = 1 ; nphen_index <= nphen ; nphen_index++)
         {
            phenout << "   " << myRandom(200);
         }
         phenout << endl;
      }

   }
   pedout.close();
   phenout.close();
}

/*
 make all tests...
*/
void make_all_tests()
{
   test_PedigreeTree();
   test_Pedigree();
   test_Person();
   test_Phenotype();
   test_Genotype();
}

/*
 make test input files...
*/
void make_test_input_files()
{
   write_test_input_files(50   ,5,2,2,"ped_50_5.txt"   ,"phen_50_5.txt");
   write_test_input_files(100  ,5,2,2,"ped_100_5.txt"  ,"phen_100_5.txt");
   write_test_input_files(500  ,5,2,2,"ped_500_5.txt"  ,"phen_500_5.txt");
   write_test_input_files(1000 ,5,2,2,"ped_1000_5.txt" ,"phen_1000_5.txt");
   write_test_input_files(5000 ,5,2,2,"ped_5000_5.txt" ,"phen_5000_5.txt");
   write_test_input_files(10000,5,2,2,"ped_10000_5.txt","phen_10000_5.txt");
   write_test_input_files(50000,5,2,2,"ped_50000_5.txt","phen_50000_5.txt");
}

/*
int main(int argc, char * argv[])
{
   make_all_tests();
   make_test_input_files();
   return 0;
}
*/
