/*
  Title     : scavenge.cpp
  Version   : 1.0
  Copyright : 2004 Tero Hiekkalinna
  Author    : Tero Hiekkalinna (tero.hiekkalinna@helsinki.fi)
  Date      : 25.1.2005
*/

#include <fstream>

#include "ScaVenge.h"

#include "Map.h"

// welcome message
void welcome()
{
   cout << "-------------------------------------------------------------------------------" << endl;
   cout << " Program Sacvenge 1.0 - February 2005" << endl;
   cout << " Copyright (C) 2005 Tero Hiekkalinna, Tero.Hiekkalinna@Helsinki.FI" << endl;
   cout << " Program for scavenging Analyze results" << endl;
   cout << "-------------------------------------------------------------------------------" << endl;
   cout << endl;
}

int main( int argc, char * argv[] )
{
  welcome();
  ScaVenge *SV = new ScaVenge();
  bool status;
  status = SV->ParseCommandLineArguments(argc,argv);
  if(status==false)
  {
    return -1;
  }
  status = SV->ReadMapFile();
  if(status==false)
  {
    return -1;
  }
  status = SV->ReadDataBaseFile();
  if(status==false)
  {
    return -1;
  }
  status = SV->WriteSummaryFile();
  if(status==false)
  {
    return -1;
  }
  status = SV->WritePlottingFile();
  if(status==false)
  {
    return -1;
  }

  return 0;
}

ScaVenge::ScaVenge()
{
  this->DataBaseFile = "database.out";
  this->MapFile = "map.dat";
  this->OutputSummaryFile = "summary.out";
  this->OutputPlottingFile ="plotdata.out";
}

bool ScaVenge::ParseCommandLineArguments(int argc, char * argv[])
{
   /*
    * Let's parse argument list...
    */
   string argument;
   for(int index = 1 ; index < argc ; index++)
   {
      argument = argv[index];

      // Database file
      if(argument == "-d" && argc >= index+1)
      {
         this->DataBaseFile = argv[index+1];
         index++;
         cout << "Database file is " << this->DataBaseFile << endl;
      }

      // Map file
      else if(argument == "-m" && argc >= index+1)
      {
         this->MapFile = argv[index+1];
         index++;
         cout << "Map file is " << this->MapFile << endl;
      }

      // Output file
      else if(argument == "-o" && argc >= index+1)
      {
         this->OutputSummaryFile = argv[index+1];
         index++;
         cout << "Output file is " << this->OutputSummaryFile << endl;
      }

      // Output file
      else if(argument == "-p" && argc >= index+1)
      {
         this->OutputPlottingFile = argv[index+1];
         index++;
         cout << "Output plotting file is " << this->OutputPlottingFile << endl;
      }

      // If argument is illegal...
      else
      {
         cout << "Error: Illegal command line argument \"" << argument << "\"" << endl;
         cout << endl;
         return false;
      }
   }

   // Everything OK!
   return true;
}

bool ScaVenge::ReadMapFile()
{
   /*
    * Map file
    */
   Map marker_pos;
   string chr, mname;
   double position;
   bool isMarker;
   char line[10001];

   cout << "Reading map file...";
   ifstream mapin(this->MapFile);
   if(!mapin)
   {
      cout << "Error: Unable to open map file " << this->MapFile << " for reading." << endl;
      return false;
   }
   mapin.getline(line,10000); // Header
   while(!mapin.eof())
   {
      mapin >> chr >> position >> mname;

      /*
       * This is needed or last record is inserted 2 times
       */
      if(mapin.eof())
         break;

      isMarker = false;
      for(unsigned int i = 0 ; i < map.size() ; i++)
      {
         if(mname == map[i].getName())
         {
            isMarker = true;
         }
      }

      if(isMarker==true)
      {
         cout << "Error: Marker \"" << mname << "\" already read (ignoring marker), double entry in map file?" << endl;
         return false;
      }
      else
      {
         marker_pos = Map(chr,position,mname);
         map.push_back(marker_pos);
      }

   }
   mapin.close();
   cout << map.size() << " markers read, done." << endl;
   return true;
}

bool ScaVenge::ReadDataBaseFile()
{
  double tmp;
  int v_locus, last_locus = -1;
  int type;
  double v_lod_hom, v_hom_theta;
  double v_lod_het, v_het_theta, v_het_alpha;
  double v_tdt;
  double v_hrr_lrt, v_hrr_2xn;
  double v_asp, v_asp_all;
  double v_nuc_lod_hom, v_nuc_hom_theta;
  double v_nuc_lod_het, v_nuc_het_theta, v_nuc_het_alpha;

  cout << "Reading database file...";
  ifstream databasein(this->DataBaseFile);
  if(!databasein)
  {
     cout << "Error: Unable to open database file " << this->DataBaseFile << " for reading." << endl;
     return false;
  }
  while(!databasein.eof())
  {
    databasein >> v_locus >> type;

    /*
     * This is needed or last record is inserted 2 times
     */
    if(databasein.eof())
       break;

    //cout << "Locus: " << v_locus << " Type: " << type << endl;

    if(v_locus != last_locus)
    {
      this->locus.push_back(v_locus);
    }

    if(type==1)
    {
      databasein >> v_lod_hom >> v_hom_theta;
      databasein >> v_lod_het >> v_het_theta >> v_het_alpha;
      if(v_hom_theta==99)
        v_hom_theta=0.5;
      if(v_het_theta==99)
        v_het_theta=0.5;
      this->lod_hom.push_back(v_lod_hom);
      this->lod_hom_theta.push_back(v_hom_theta);
      this->lod_het.push_back(v_lod_het);
      this->lod_het_theta.push_back(v_het_theta);
      this->lod_het_alpha.push_back(v_het_alpha);
    }
    else if(type==2)
    {
      databasein >> tmp >> v_tdt >> tmp;
      this->tdt.push_back(v_tdt);
    }
    else if(type==3)
    {
      databasein >> tmp >> v_hrr_lrt >> tmp;
      databasein >> tmp >> v_hrr_2xn;
      this->hrr_lrt.push_back(v_hrr_lrt);
      this->hrr_2xn.push_back(v_hrr_2xn);
    }
    else if(type==4)
    {
      databasein >> v_asp >> tmp >> tmp;
      databasein >> v_asp_all >> tmp >> tmp;
      this->asp.push_back(v_asp);
      this->asp_all.push_back(v_asp_all);
    }
    else if(type==5)
    {
      databasein >> v_nuc_lod_hom >> v_nuc_hom_theta;
      databasein >> v_nuc_lod_het >> v_nuc_het_theta >> v_nuc_het_alpha;
      if(v_nuc_hom_theta==99)
        v_nuc_hom_theta=0.5;
      if(v_nuc_het_theta==99)
        v_nuc_het_theta=0.5;
      this->nuc_lod_hom.push_back(v_nuc_lod_hom);
      this->nuc_lod_hom_theta.push_back(v_nuc_hom_theta);
      this->nuc_lod_het.push_back(v_nuc_lod_het);
      this->nuc_lod_het_theta.push_back(v_nuc_het_theta);
      this->nuc_lod_het_alpha.push_back(v_nuc_het_alpha);
    }
    else
    {
      cout << "Error in result type " << type << ", exiting..." << endl;
      return false;
    }

    last_locus = v_locus;

  }
  databasein.close();
  cout << "done" << endl;
  return true;
}

bool ScaVenge::WriteSummaryFile()
{
  cout << "Writing summary file..." << this->OutputSummaryFile << "...";
  fstream summary(this->OutputSummaryFile,ios_base::out);
  if(summary==NULL)
  {
     cout << "Error: Unable to open " << this->OutputSummaryFile << " for writing!" << endl;
     return false;
  }

  summary << "-------------------------------------------------------------------------------" << endl;
  summary << " ANALYZE summary report from AUTOGSCAN" << endl;
  summary << " Copyright (C) 2004-2005 Tero Hiekkalinna, Tero.Hiekkalinna@Helsinki.FI" << endl;
  summary << "-------------------------------------------------------------------------------" << endl;
  summary << endl;

  if(map.size()>0)
  {
    summary << "Chromosome " << this->map[0].getChr() << endl;
    summary << endl;
  }

  summary << "Parametric Linkage analysis summary - All on Lod Score Scale" << endl;
  summary << endl;

  summary.setf(ios_base::fixed,ios_base::floatfield);

  summary << "        Name      LodHom (theta)       LodHet (theta,alpha)" << endl;
  for(unsigned int i = 0 ; i < this->locus.size() ; i++)
  {
    summary.width(12);
    summary.setf(ios::right);
    summary << this->map[i].getName();
    summary.setf(ios::right);

    if(lod_hom.size()!=0 && i < lod_hom.size())
    {
      summary.width(12);
      summary.precision(6);
      summary << lod_hom[i];
      summary << " (";
      summary.width(4);
      summary.precision(2);
      summary << lod_hom_theta[i];
      summary << ")  ";
    }
    else
    {
      summary << "N/A      (N/A)   ";
    }

    if(lod_het.size()!=0 && i < lod_het.size())
    {
      summary.width(12);
      summary.precision(6);
      summary << lod_het[i];
      summary.precision(2);
      summary << "  (" << lod_het_theta[i] << "," << lod_het_alpha[i] << ")  ";
    }
    else
    {
      summary << "N/A       (N/A,N/A)   ";
    }
    summary << endl;
  }

  summary << endl;

  summary << "        Name   NucLodHom (theta)    NucLodHet (theta,alpha)" << endl;
  for(unsigned int i = 0 ; i < this->locus.size() ; i++)
  {
    summary.width(12);
    summary.setf(ios::right);
    summary << this->map[i].getName();
    summary.setf(ios::right);

    if(nuc_lod_hom.size()!=0 && i < nuc_lod_hom.size())
    {
      summary.width(12);
      summary.precision(6);
      summary << nuc_lod_hom[i];
      summary << " (";
      summary.width(4);
      summary.precision(2);
      summary << nuc_lod_hom_theta[i];
      summary << ")  ";
    }
    else
    {
      summary << "N/A       (N/A)   ";
    }

    if(nuc_lod_het.size()!=0 && i < nuc_lod_het.size())
    {
      summary.width(12);
      summary.precision(6);
      summary << nuc_lod_het[i];
      summary.precision(2);
      summary << "  (" << nuc_lod_het_theta[i] << "," << nuc_lod_het_alpha[i] << ") ";
    }
    else
    {
      summary << "N/A       (N/A,N/A)";
    }
    summary << endl;
  }


  summary << endl;
  summary << "Sibpair Linkage analysis summary - All on Lod Score Scale" << endl;
  summary << endl;

  summary << "        Name         ASP     All-ASP" << endl;
  for(unsigned int i = 0 ; i < this->locus.size() ; i++)
  {
    summary.width(12);
    summary.setf(ios::right);
    summary << this->map[i].getName();
    summary.setf(ios::right);

    if(asp.size()!=0 && i < asp.size())
    {
      summary.width(12);
      summary.precision(6);
      summary << asp[i];
    }
    else
    {
      summary << "N/A       ";
    }

    if(asp_all.size()!=0 && i < asp_all.size())
    {
      summary.width(12);
      summary.precision(6);
      summary << asp_all[i];
    }
    else
    {
      summary << "N/A       ";
    }

    summary << endl;
  }

  summary << endl;
  summary << "Assosiation analysis summary - results are p-values" << endl;
  summary << endl;

  summary << "        Name     HRR-LRT     HRR-2xn         TDT" << endl;
  for(unsigned int i = 0 ; i < this->locus.size() ; i++)
  {
    summary.setf(ios::right);
    summary.width(12);
    summary << this->map[i].getName();
    summary.setf(ios::right);

    if(hrr_lrt.size()!=0 && i < hrr_lrt.size())
    {
      summary.width(12);
      summary.precision(6);
      summary << hrr_lrt[i];
    }
    else
    {
      summary << "N/A        ";
    }

    if(hrr_2xn.size()!=0 && i < hrr_2xn.size())
    {
      summary.width(12);
      summary.precision(6);
      summary << hrr_2xn[i];
    }
    else
    {
      summary << "N/A        ";
    }

    if(tdt.size()!=0 && i < tdt.size())
    {
      summary.width(12);
      summary.precision(6);
      summary << tdt[i];
    }
    else
    {
      summary << "N/A       ";
    }
    summary << endl;
  }

  summary << endl;
  summary << "N/A = Not Available" << endl;
  summary << endl;

  // Highest

  summary << "Summary of summary..." << endl;
  summary << endl;

  double highest=0.0;
  int highest_index;
  for(unsigned int i = 0 ; i < this->lod_hom.size() ; i++)
  {
    if(lod_hom[i] >= highest)
    {
      highest = lod_hom[i];
      highest_index = i;
    }
  }
  if(this->lod_hom.size()>0)
  {
    summary << "Highest LOD score under homogeneity   was " << highest << " (" << this->map[highest_index].getName() << ")" << endl;
  }
  highest=0.0;
  for(unsigned int i = 0 ; i < this->lod_het.size() ; i++)
  {
    if(lod_het[i] >= highest)
    {
      highest = lod_het[i];
      highest_index = i;
    }
  }
  if(this->lod_het.size()>0)
  {
    summary << "Highest LOD score under heterogeneity was " << highest << " (" << this->map[highest_index].getName() << ")" << endl;
  }
  highest=0.0;
  for(unsigned int i = 0 ; i < this->nuc_lod_hom.size() ; i++)
  {
    if(nuc_lod_hom[i] >= highest)
    {
      highest = nuc_lod_hom[i];
      highest_index = i;
    }
  }
  if(this->nuc_lod_hom.size()>0)
  {
    summary << "Highest nuclear pedigree LOD score under homogeneity   was " << highest << " (" << this->map[highest_index].getName() << ")" << endl;
  }
  highest=0.0;
  for(unsigned int i = 0 ; i < this->nuc_lod_het.size() ; i++)
  {
    if(nuc_lod_het[i] >= highest)
    {
      highest = nuc_lod_het[i];
      highest_index = i;
    }
  }
  if(this->nuc_lod_het.size()>0)
  {
    summary << "Highest nuclear pedigree LOD score under heterogeneity was " << highest << " (" << this->map[highest_index].getName() << ")" << endl;
  }
  highest=0.0;
  for(unsigned int i = 0 ; i < this->asp.size() ; i++)
  {
    if(asp[i] >= highest)
    {
      highest = asp[i];
      highest_index = i;
    }
  }
  if(this->asp.size()>0)
  {
    summary << "Highest SIBPAIR LOD score     was " << highest << " (" << this->map[highest_index].getName() << ")" << endl;
  }
  highest=0.0;
  for(unsigned int i = 0 ; i < this->asp_all.size() ; i++)
  {
    if(asp_all[i] >= highest)
    {
      highest = asp_all[i];
      highest_index = i;
    }
  }
  if(this->asp_all.size()>0)
  {
    summary << "Highest All-SIBPAIR LOD score was " << highest << " (" << this->map[highest_index].getName() << ")" << endl;
  }

  // lowest
  double lowest=1.0;
  int lowest_index;
  for(unsigned int i = 0 ; i < this->hrr_lrt.size() ; i++)
  {
    if(hrr_lrt[i] <= lowest)
    {
      lowest = hrr_lrt[i];
      lowest_index = i;
    }
  }
  if(this->hrr_lrt.size()>0)
  {
    summary << "Lowest HRR-LRT p-value was " << lowest << " (" << this->map[lowest_index].getName() << ")" << endl;
  }
  lowest=1.0;
  for(unsigned int i = 0 ; i < this->hrr_2xn.size() ; i++)
  {
    if(hrr_2xn[i] <= lowest)
    {
      lowest = hrr_2xn[i];
      lowest_index = i;
    }
  }
  if(this->hrr_2xn.size()>0)
  {
    summary << "Lowest HRR-2xn p-value was " << lowest << " (" << this->map[lowest_index].getName() << ")" << endl;
  }
  lowest=1.0;
  for(unsigned int i = 0 ; i < this->tdt.size() ; i++)
  {
    if(tdt[i] <= lowest)
    {
      lowest = tdt[i];
      lowest_index = i;
    }
  }
  if(this->tdt.size()>0)
  {
    summary << "Lowest TDT     p-value was " << lowest << " (" << this->map[lowest_index].getName() << ")" << endl;
  }

  summary << endl;


  summary.close();
  cout << "done." << endl;
  return true;
}

bool ScaVenge::WritePlottingFile()
{
  cout << "Writing plotting file..." << this->OutputPlottingFile << "...";
  fstream plotout(this->OutputPlottingFile,ios_base::out);
  if(plotout==NULL)
  {
     cout << "Error: Unable to open " << this->OutputPlottingFile << " for writing!" << endl;
     return false;
  }

  plotout.setf(ios_base::fixed,ios_base::floatfield);
  plotout.precision(6);
  plotout << "# Data for plotting with Gnuplot" << endl;
  plotout << "# Chr Name Position LodHom theta LodHet theta alpha NucLodHom theta NucLodHet theta alpha ASP All-ASP HRR-LRT HRR-2xn TDT" << endl;
  for(unsigned int i = 0 ; i < this->locus.size() ; i++)
  {
    plotout << this->map[i].getChr() << " " << this->map[i].getName() << " " << this->map[i].getPosition() << " ";

    if(lod_hom.size()!=0 && i < lod_hom.size())
    {
      plotout << lod_hom[i] << " " << lod_hom_theta[i] << " ";
    }
    else
    {
      plotout << "- - ";
    }

    if(lod_het.size()!=0 && i < lod_het.size())
    {
      plotout << lod_het[i] << " " << lod_het_theta[i] << " " << lod_het_alpha[i] << " ";
    }
    else
    {
      plotout << "- - - ";
    }

    if(nuc_lod_hom.size()!=0 && i < nuc_lod_hom.size())
    {
      plotout << nuc_lod_hom[i] << " " << nuc_lod_hom_theta[i] << " ";
    }
    else
    {
      plotout << "- - ";
    }

    if(nuc_lod_het.size()!=0 && i < nuc_lod_het.size())
    {
      plotout << nuc_lod_het[i] << " " << nuc_lod_het_theta[i] << " " << nuc_lod_het_alpha[i] << " ";
    }
    else
    {
      plotout << " - - ";
    }

    if(asp.size()!=0 && i < asp.size())
    {
      plotout << asp[i] << " ";
    }
    else
    {
      plotout << "- ";
    }

    if(asp_all.size()!=0 && i < asp_all.size())
    {
      plotout << asp_all[i] << " ";
    }
    else
    {
      plotout << "- ";
    }

    if(hrr_lrt.size()!=0 && i < hrr_lrt.size())
    {
      plotout << hrr_lrt[i] << " ";
    }
    else
    {
      plotout << "- ";
    }

    if(hrr_2xn.size()!=0 && i < hrr_2xn.size())
    {
      plotout << hrr_2xn[i] << " ";
    }
    else
    {
      plotout << "- ";
    }

    if(tdt.size()!=0 && i < tdt.size())
    {
      plotout << tdt[i] << " ";
    }
    else
    {
      plotout << "- ";
    }

    plotout << endl;
  }
  plotout.close();
  cout << "done." << endl;
  return true;
}
